@@ -7,18 +7,14 @@ import pandas as pd
 import re
 
 
-def __src(x):
-    root = os.path.dirname(__file__)
-    return os.path.abspath(os.path.join(root, x))
+def _get_version():
+    import ph._version
+
+    return ph._version.__version__
 
 
-def __read_file(fname):
-    with open(__src(fname), "r") as fin:
-        return "".join([e.strip() for e in fin.readlines()]).strip()
-
-
-def _version():
-    return __read_file("version.txt")
+def print_version():
+    print(_get_version())
 
 
 # Command line parsing of (1) --abc and (2) --abc=def
@@ -167,9 +163,46 @@ def dataset(dset=None):
 
 
 @register
-def dropna():
-    """Remove rows with N/A values"""
-    pipeout(pipein().dropna())
+def dropna(axis=0, how="any", thresh=None):
+    """Remove rows (or columns) with N/A values.
+
+    Argument: --axis=0
+    Defaults to axis=0 (columns), use --axis=1 to remove rows.
+
+    Argument: --how=any
+    Defaults to how='any', which removes columns (resp. rows) containing
+    nan values.  Use how='all' to remove columns (resp. rows) containing
+    only nan values.
+
+    Argument: --thresh=5
+    If --thresh=x is specified, will delete any column (resp. row) with
+    fewer than x non-na values.
+
+    Usage: cat a.csv | ph dropna
+           cat a.csv | ph dropna --axis=1    # for row-wise
+           cat a.csv | ph dropna --thresh=5  # keep cols with >= 5 non-na
+           cat a.csv | ph dropna --how=all   # delete only if all vals na
+
+    """
+    try:
+        axis = int(axis)
+        if axis not in (0, 1):
+            exit("ph dropna --axis=0 or --axis=1, not {}".format(axis))
+    except ValueError:
+        exit("ph dropna --axis=0 or --axis=1, not {}".format(axis))
+
+    if thresh is not None:
+        try:
+            thresh = int(thresh)
+        except ValueError:
+            exit("ph dropna --thresh=0 or --thresh=1, not {}".format(thresh))
+
+    df = pipein()
+    try:
+        df = df.dropna(axis=axis, how=how, thresh=thresh)
+    except Exception as err:
+        exit(err)
+    pipeout(df)
 
 
 def pipeout(df, sep=",", index=False, *args, **kwargs):
@@ -195,6 +228,46 @@ def pipein(ftype="csv"):
         return pd.DataFrame()
 
 
+@register
+def fillna(value=None, method=None, limit=None):
+    """Fill na values with a certain value or method, at most `limit` many.
+
+    Takes either a value, or a method using (e.g.) --method=ffill.
+
+    Argument: value
+    If provided, replaces all N/A values with prescribed value.
+
+    Argument: --method=pad
+    If provided, value cannot be provided.  Allowed methods are
+    backfill, bfill, pad, ffill
+
+    Argument: --limit=x
+    If provided, limits number of consecutive N/A values to fill.
+
+
+    Usage: cat a.csv | ph fillna 999.75
+           cat a.csv | ph fillna -1
+           cat a.csv | ph fillna --method=pad
+           cat a.csv | ph fillna --method=pad --limit=5
+
+    """
+    if limit is not None:
+        try:
+            limit = int(limit)
+        except ValueError:
+            exit("--limit=x must be an integer, not {}".format(limit))
+    METHODS = ("backfill", "bfill", "pad", "ffill")
+    if method is not None:
+        if method not in METHODS:
+            exit("method must be one of {}, not {}".format(METHODS, method))
+        pipeout(pipein().fillna(method=method, limit=limit))
+    elif value is not None:
+        value = __tryparse(value)
+        pipeout(pipein().fillna(value=value, limit=limit))
+    else:
+        exit("'ph dropna' needs exactly one of value and method")
+
+
 @register
 def query(expr):
     """Using pandas queries.
@@ -360,6 +433,15 @@ def eval(expr):
 
 @register
 def normalize(col=None):
+    """Normalize a column or an entire dataframe.
+
+    Usage: cat a.csv | ph normalize
+           cat a.csv | ph normalize x
+
+
+    Warning:  This is probably not what you want.
+
+    """
     df = pipein()
     if col is None:
         df = (df - df.min()) / (df.max() - df.min())
@@ -369,14 +451,37 @@ def normalize(col=None):
 
 
 @register
-def date(col):
+def date(col=None, unit=None, origin="unix"):
+    """Assemble datetime from multiple columns or from one column
+
+    --unit can be D, s, us, ns (defaults to ns, ns from origin)
+
+    --origin can be unix, julian, or time offset, e.g. '2000-01-01'
+
+    Usage: cat a.csv | ph date x
+           cat a.csv | ph date x --unit=s --origin="1984-05-17 09:30"
+
+    """
     df = pipein()
-    df[col] = pd.to_datetime(df[col])
+    try:
+        if col is None:
+            df = pd.to_datetime(df)
+        else:
+            df[col] = pd.to_datetime(df[col], unit=unit, origin=origin)
+    except Exception as err:
+        exit(err)
+
     pipeout(df)
 
 
 @register
 def describe():
+    """Run DataFrame's describe method.
+
+    The result is NOT tabular data, so pipeline ends.
+
+    Usage: cat a.csv | ph describe
+    """
     try:
         print(pipein().describe())
     except ValueError as err:
@@ -385,6 +490,12 @@ def describe():
 
 @register
 def info():
+    """Run DataFrame's info method.
+
+    The result is NOT tabular data, so pipeline ends.
+
+    Usage: cat a.csv | ph info
+    """
     print(pipein().info())
 
 
@@ -663,6 +774,10 @@ def columns(*cols, **kwargs):
             if col.endswith(q) and col not in cols:
                 cols.append(col)
 
+    for col in cols:
+        if col not in df.columns:
+            exit("No such column {}".format(col))
+
     if not cols and not kwargs:
         print("\n".join(list(df.columns)))
     else:
@@ -733,7 +848,7 @@ def main():
         exit("Usage: ph command [args]\n       ph help")
     cmd = sys.argv[1]
     if cmd in ("-v", "--version"):
-        print(_version())
+        print_version()
         exit()
     if cmd not in COMMANDS:
         exit("Unknown command {}.".format(cmd))
@@ -753,7 +868,10 @@ def main():
             kwarg[k] = v
         else:
             args.append(a)
-    COMMANDS[cmd](*args, **kwarg)
+    try:
+        COMMANDS[cmd](*args, **kwarg)
+    except TypeError as err:
+        exit(err)
 
 
 if __name__ == "__main__":
